'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.start = start;
exports.stop = stop;
exports.request = request;
exports.addEntries = addEntries;

var _actions = require('./actions');

var _reducer = require('./reducer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function start(_ref, action, next) {
    var getState = _ref.getState,
        dispatch = _ref.dispatch;

    var state = getState();
    var pollingName = action.meta.pollingName;

    var _getPollingState = (0, _reducer.getPollingState)(state, pollingName),
        isActive = _getPollingState.isActive;

    if (isActive) {
        return false;
    }

    next(action);

    var meta = (0, _extends3.default)({}, action.meta, { initialPolling: true });
    var requestAction = (0, _actions.createAction)(_actions.actionTypes.request, meta);
    dispatch(requestAction);

    return true;
}

function stop(_, action, next) {
    next(action);
    return true;
}

function request(_ref2, action) {
    var getState = _ref2.getState,
        dispatch = _ref2.dispatch;

    var state = getState();
    var pollingName = action.meta.pollingName;

    var _getPollingState2 = (0, _reducer.getPollingState)(state, pollingName),
        isActive = _getPollingState2.isActive,
        requestPayload = _getPollingState2.requestPayload;

    if (!isActive) {
        return false;
    }

    var _action$meta = action.meta,
        callbacks = _action$meta.callbacks,
        pollingInterval = _action$meta.pollingInterval,
        initialPolling = _action$meta.initialPolling;

    var isInitialPolling = initialPolling === true && typeof callbacks.initialPolling === 'function';
    var pollingFunc = isInitialPolling ? callbacks.initialPolling : callbacks.polling;
    return Promise.resolve(pollingFunc.apply(undefined, (0, _toConsumableArray3.default)(requestPayload).concat([getState]))).then(function (data) {
        var entries = isInitialPolling ? data : [data];
        var addEntriesAction = (0, _actions.createAction)(_actions.actionTypes.addEntries, action.meta, entries);
        dispatch(addEntriesAction);
    }, function (err) {
        return err;
    } // If exception during polling - do nothing
    ).then(function () {
        setTimeout(function () {
            var meta = (0, _extends3.default)({}, action.meta, { initialPolling: false });
            var requestAction = (0, _actions.createAction)(_actions.actionTypes.request, meta);
            dispatch(requestAction);
        }, pollingInterval);
    });
}

function addEntries(_ref3, action, next) {
    var getState = _ref3.getState;
    var shouldAddEntry = action.meta.callbacks.shouldAddEntry;

    var entries = action.payload;

    if (!Array.isArray(entries)) {
        throw new Error('Payload for ' + _actions.actionTypes.addEntries + ' must be array of entries');
    }

    if (typeof shouldAddEntry === 'function') {
        entries = entries.filter(function (entry) {
            return shouldAddEntry(getState, entry);
        });
    }

    if (entries.length > 0) {
        next((0, _extends3.default)({}, action, {
            payload: entries
        }));
    }

    return entries.length;
}